import json
import tempfile
from logging import Logger, getLogger
from pathlib import Path

import requests
from apistar.exceptions import ErrorResponse
from teklia_toolbox.requests import download_file

logger: Logger = getLogger(__name__)


def download_files(api_client, datafiles):
    temp_path = tempfile.mkdtemp()
    files = []
    for datafile_id in datafiles:
        df = api_client.request("RetrieveDataFile", id=datafile_id)
        path = Path(temp_path) / datafile_id

        logger.info("Downloading file {} ({})".format(df["name"], df["id"]))
        try:
            download_file(df["s3_url"], path)
        except requests.RequestException as e:
            logger.error(
                "Failed downloading file {} ({}): {}".format(
                    df["name"], df["id"], str(e)
                )
            )
            continue

        df["local_path"] = path
        files.append(df)

    return files


def dump_json(elements, dir, path=None, filename="elements.json"):
    file_path = path and Path(path, filename)
    if not file_path:
        file_path = dir / filename
    assert len(elements), f"No elements could be written to {file_path}"

    with file_path.open("w") as f:
        json.dump(elements, f, indent=4)


def delete_files(api_client, files):
    """
    Delete DataFiles once they are imported.
    Deletion errors will only show warnings instead of making the import fail.
    """
    for datafile in files:
        try:
            api_client.request("DestroyDataFile", id=datafile["id"])
        except ErrorResponse as e:
            logger.warning(f'Could not delete file {datafile["name"]}: {e}')
